@@ -11,6 +11,7 @@
 # 2014-03-14: error message for non-factor, non-numeric predictor
 # 2014-07-08: if no numeric predictor, partial residuals suppressed with warning rather than an error
 # 2014-10-09: namespace fixes. J. Fox
+# 2015-04-08: added setStrip(), restoreStrip(). J. Fox
 
 has.intercept <- function(model, ...) any(names(coefficients(model))=="(Intercept)")
 
@@ -590,4 +591,34 @@ is.factor.predictor <- function(predictor, model) {
 
 is.numeric.predictor <- function(predictor, model) {
     is.null(model$xlevels[[predictor]])
+}
+
+# manage lattice strips
+
+setStrip <- function(bg=3, fg="black", clip=c("off", "on")){
+    clip <- match.arg(clip)
+    bg.save <- strip.background <- trellis.par.get("strip.background")
+    if (is.numeric(bg) && length(bg) == 1){
+        if (bg <= 0) stop("bg should be a positive integer or vector of colors")
+        bg <- gray(seq(.95, .5, length=round(bg)))
+    }
+    strip.background$col <- bg
+    fg.save <- strip.shingle <- trellis.par.get("strip.shingle")
+    trellis.par.set("strip.background", strip.background)
+    if (length(fg) != 1 && length(fg) != length(bg)) 
+        stop("lengths of fg and bg incompatible")
+    strip.shingle$col <- fg
+    trellis.par.set("strip.shingle", strip.shingle)
+    clip.save <- .clip <- trellis.par.get("clip")
+    .clip$strip <- clip
+    trellis.par.set("clip", .clip)
+    invisible(list(strip.background=bg.save, strip.shingle=fg.save, clip=clip.save))
+}
+
+restoreStrip <- function(saved){
+    if (!identical(names(saved), c("strip.background", "strip.shingle", "clip")))
+        stop("argument saved does not contain strip parameters")
+    trellis.par.set("strip.background", saved$strip.background)
+    trellis.par.set("strip.shingle", saved$strip.shingle)
+    trellis.par.set("clip", saved$clip)
 }
\ No newline at end of file
